// 
// このコードは、https://demo.mescius.jp/diodocs/pdf のサンプルの一部として提供されています。
// © MESCIUS inc. All rights reserved.
// 
using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using GrapeCity.Documents.Drawing;
using GrapeCity.Documents.Text;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Pdf.Annotations;
using GrapeCity.Documents.Pdf.Actions;
using GrapeCity.Documents.Html;
using Color = System.Drawing.Color;
using Font = GrapeCity.Documents.Text.Font;
using RectangleF = System.Drawing.RectangleF;
using SizeF = System.Drawing.SizeF;

namespace DsPdfWeb.Demos.Common
{
    public static class Util
    {
        public const int LargeDocumentIterations = 1000;
        private static int s_gen = 0;
        private static DateTime s_fixedTime = new DateTime(1989, 6, 21, 1, 23, 45, DateTimeKind.Utc);

        private static readonly string[] s_words = new[]
            {
                "lorem", "ipsum", "dolor", "sit", "amet", "consectetur", "adipiscing", "elit", "mauris", "id", "volutpat",
                "tellus", "ullamcorper", "sem", "praesent", "et", "ante", "laoreet", "lobortis", "nunc", "congue",
                "nisi", "donec", "ac", "tempus", "sed", "feugiat", "pulvinar", "pharetra", "turpis", "nonummy",
                "at", "felis", "eget", "molestie", "euismod", "non", "aliquet", "diam", "proin", "mi", "nibh",
                "massa", "tincidunt", "ut", "dolore", "magna", "aliquam", "erat",
            };
        // サンプル文字列  30件
        private static readonly string[] s_wordsJP = new[]
            {
            "うみ","やま","そら","にじ","くも",
            "Pen","Desk","Book","Chair" , "Text",
            "壱弐参四五", "67890", "平成30年10月17日","2018/10/17","〒981-3205",
            "!", "?", "(", ")", "+",
            "！", "？", "【", "】"," ※",
            "鯵","鯖","鮭","鰯","鮪","鮑","鮃","鰈","鯛","鰹",
            "ウニ","イクラ","アナゴ","エビ","タコ","アサリ","イカ","アユ","ナマズ","サメ"
            };
        // 宮沢賢治 「雨ニモマケズ（現代文語表記）」 30件
        private static readonly string[] s_amenimo = new[]
        {
            "雨にも負けず", "風にも負けず", "雪にも夏の暑さにも負けぬ", "丈夫なからだをもち", "慾はなく",
            "決して怒らず", "いつも静かに笑っている", "一日に玄米四合と", "味噌と少しの野菜を食べ", "あらゆることを",
            "自分を勘定に入れずに", "よく見聞きし分かり", "そして忘れず", "野原の松の林の陰の", "小さな萱ぶきの小屋にいて",
            "東に病気の子供あれば", "行って看病してやり", "西に疲れた母あれば", "行ってその稲の束を負い","南に死にそうな人あれば",
            "行ってこわがらなくてもいいといい", "北に喧嘩や訴訟があれば", "つまらないからやめろといい", "日照りの時は涙を流し", "寒さの夏はおろおろ歩き",
            "みんなにでくのぼーと呼ばれ", "褒められもせず", "苦にもされず", "そういうものに", "わたしはなりたい",
        };

        public static bool NoRandom { get; set; } = false;

        public static Random NewRandom()
        {
            if (NoRandom)
                return new Random(0);
            else
            return new Random((int)DateTime.Now.Ticks + Interlocked.Increment(ref s_gen));
        }

        public static DateTime TimeNow()
        {
            if (NoRandom)
                return s_fixedTime;
            else
                return DateTime.Now;
        }

        private static string s_browserBasePath = null;
        private static readonly object s_browserBasePath_lock = new object();

        // 既知の場所からブラウザを使用する場合に設定します。
        public static void SetBrowserBasePath(string browserBasePath)
        {
            lock (s_browserBasePath_lock)
                s_browserBasePath = browserBasePath;
        }

        // DsHtml V6J以降：このメソッドは、オンラインサンプルだけでなく、ユーザーがダウンロードした
        // すべてのサンプルでも使用できるひとつのターゲットロケーションを提供するため、
        // 各サンプルが独自にブラウザをダウンロードしてインストールするのではなく、
        // 複数のサンプルが同じブラウザのインストールを共有できます。
        // Chrome がインストールされているシステムでは、BrowserFetcher.GetSystemChromePath() を呼び出すと、
        // インストールされている Chrome ブラウザが使用されます。
        public static string GetBrowserPath()
        {
            if (s_browserBasePath == null)
            {
                var systemPath = BrowserFetcher.GetSystemChromePath();
                if (systemPath != null)
                    return systemPath;
            }
            var path = s_browserBasePath ?? Path.Combine(Path.GetTempPath(), ".gc-chromium");
            var bf = new BrowserFetcher() { DestinationFolder = path };
            return bf.GetDownloadedPath();
        }

        // DsHtml V6J:
        public static GcHtmlBrowser NewHtmlBrowser()
        {
            var path = GetBrowserPath();
            try
            {
                if (RuntimeInformation.IsOSPlatform(OSPlatform.Linux))
                    return new GcHtmlBrowser(path, new LaunchOptions { RunWithNoSandbox = true });
                else
                    return new GcHtmlBrowser(path);
            }
            catch (Exception ex)
            {
                if (path == BrowserFetcher.GetSystemChromePath())
                    throw;
                else
                    throw new Exception("Exception occurred when launching the browser (see inner exception for details), " +
                        $"the browser installation in {Path.GetDirectoryName(path)} may be corrupt. Please remove it and try again.", ex);
            }
        }

        // https://stackoverflow.com/questions/4286487/is-there-any-lorem-ipsum-generator-in-c
        public static string LoremIpsum(
            int numParagraphs = 5,
            int minSentences = 5,
            int maxSentences = 10,
            int minWords = 5,
            int maxWords = 30)
        {
            int commaMax = 10;
            var rnd = NewRandom();
            int numSentences = rnd.Next(maxSentences - minSentences) + minSentences + 1;
            int numWords = rnd.Next(maxWords - minWords) + minWords + 1;

            int wdslen = s_words.Length;
            var result = new StringBuilder();
            for (int p = 0; p < numParagraphs; p++)
            {
                for (int s = 0; s < numSentences; s++)
                {
                    for (int w = 0; w < numWords; w++)
                    {
                        var word = s_words[rnd.Next(wdslen)];
                        if (w == 0)
                        {
                            result.Append(char.ToUpper(word[0]));
                            result.Append(word.Substring(1));
                        }
                        else
                        {
                            if (rnd.Next(commaMax) == 0)
                                result.Append(',');
                            result.Append(' ');
                            result.Append(word);
                        }
                    }
                    result.Append(". ");
                }
                result.Append("\r\n");
            }
            return result.ToString();
        }
        public static string getString_ja(
            int numFullSentence = 1,
            int numSentence = 0,
            int numParagraphs = 5,
            int minSentences = 2,
            int maxSentences = 30,
            bool sentenceset = true
            )
        {
            List<string> wordset = AmenimoMakezu();
            string spChar = "、";
            if (sentenceset == false)
            {
                wordset = s_wordsJP.ToList();
                spChar = " ";
            }
            var rand = NewRandom();
            int wdslen = wordset.ToArray().Length;
            maxSentences = (maxSentences > wdslen) ? wdslen : maxSentences;

            StringBuilder result = new StringBuilder();
            for (int p = 0; p < numParagraphs; p++)
            {
                int numSentences = wdslen;
                if (p > numFullSentence - 1)
                {
                    numSentences = rand.Next(minSentences,maxSentences);
                }
                for (int s = 0; s < numSentences; s++)
                {
                    string word;
                    if (sentenceset == false)
                    {
                        word = wordset[rand.Next(wdslen)];
                    }
                    else
                    {
                        word = wordset[s];
                    }

                    if (s > 0)
                    {
                        if (numSentence == 0)
                        {
                            result.Append(spChar);
                        }
                        else
                        {
                            result.Append('\n');
                        }
                    }
                    result.Append(word);
                }
                result.Append("。\n");
                result.Append("\n");
            }
            result.Append("\r\n");
            return result.ToString();

        }
        // LoremIpsum メソッド で利用するためのサンプル文字列を取得
        public static List<string> LoremWords()
        {
            return s_words.ToList();
        }
        // サンプルの日本語文字列を取得
        public static List<string> AmenimoMakezu()
        {
            return s_amenimo.ToList();
        }

        // https://stackoverflow.com/questions/273313/randomize-a-listt
        public static void Shuffle<T>(this IList<T> list)
        {
            var rnd = NewRandom();
            int n = list.Count;
            while (n > 1)
            {
                n--;
                int k = rnd.Next(n + 1);
                T value = list[k];
                list[k] = list[n];
                list[n] = value;
            }
        }

        /// <summary>
        /// ドキュメントにメモを追加します。色のついた背景の上にテキストが表示されます。
        /// サンプルドキュメントに情報を挿入するために使用します。
        /// </summary>
        /// <param name="text">描画するテキスト</param>
        /// <param name="page">描画するページ</param>
        /// <param name="bounds">テキストの境界。NULLの場合、1インチの余白があるページ全体が使用されます。
        /// NULLではなく高さが0の場合、余白を指定するために使用されます。</param>
        /// <returns>メモが占める実際の矩形</returns>
        public static RectangleF AddNote(string text, Page page, RectangleF? bounds = null)
        {
            var g = page.Graphics;
            var tl = g.CreateTextLayout();
            var pad = g.Resolution / 8f;
            tl.DefaultFormat.Font = StandardFonts.Helvetica;
            var ipag = Path.Combine("Resources", "Fonts", "ipag.ttc");
            if (File.Exists(ipag))
                tl.DefaultFormat.Font.AddLinkedFont(Font.FromFile(ipag));
            tl.DefaultFormat.FontSize = 12;
            if (!bounds.HasValue)
            {
                var margin = g.Resolution;
                bounds = new RectangleF(margin, margin, page.Size.Width - margin * 2, page.Size.Height - margin * 2);
            }
            else if (bounds.Value.Height == 0)
                bounds = new RectangleF(bounds.Value.Location, new SizeF(bounds.Value.Width, page.Size.Height - bounds.Value.Top * 2));

            tl.MaxWidth = bounds.Value.Width;
            if (bounds.Value.Height > 0)
                tl.MaxHeight = bounds.Value.Height;
            tl.MarginAll = pad;

            var texts = Regex.Split(text, "(https?://[\\S]+)", RegexOptions.IgnoreCase);
            var runs = new List<TextRun>();
            var urls = new List<string>();
            foreach (var txt in texts)
            {
                if (txt.StartsWith("http", StringComparison.OrdinalIgnoreCase))
                {
                    urls.Add(txt);
                    runs.Add(tl.Append(txt, new TextFormat(tl.DefaultFormat)
                    { ForeColor = Color.Blue, Underline = true, TextRunAsCluster = true, }));
                }
                else
                    tl.Append(txt);
            }
            tl.PerformLayout(true);
            for (int i = 0; i < runs.Count; ++i)
            {
                var run = runs[i];
                var rects = tl.GetTextRects(run.CodePointIndex, run.CodePointCount);
                foreach (var r in rects)
                {
                    var rc = r.ToRectangleF();
                    rc.Offset(bounds.Value.Location);
                    page.Annotations.Add(new LinkAnnotation(rc, new ActionURI(urls[i])));
                }
            }

            var rect = tl.ContentRectangle;
            rect.Offset(bounds.Value.Location);
            rect.Inflate(pad, pad);
            g.FillRectangle(rect, Color.FromArgb(213, 221, 240));
            g.DrawRectangle(rect, Color.FromArgb(59, 92, 170), 0.5f);
            g.DrawTextLayout(tl, bounds.Value.Location);

            return rect;
        }

        /// <summary>
        /// ディスクファイルから画像を読み込みます
        /// <para>
        /// このメソッドは、Windows Imaging Component フレームワークを介して動作する
        /// GrapeCity.Documents.Drawing.Windows.WicImage を使用することで、
        /// Windows システム上でより良いパフォーマンスを提供します。
        /// WicImage がサポートされていない場合、このメソッドはデフォルトで
        /// プラットフォーム非依存の Image.FromFile() メソッドになります。
        /// </para>
        /// </summary>
        /// <param name="path">画像のパス</param>
        /// <returns>画像オブジェクト</returns>
        public static IImage ImageFromFile(string path)
        {
            if (GrapeCity.Documents.Imaging.Windows.GcWicBitmap.IsSupported)
                return new GrapeCity.Documents.Imaging.Windows.GcWicBitmap(path);
            return Image.FromFile(path);
        }

        /// <summary>
        /// フォントを読み込みます
        /// <para>
        /// リソースからフォントを読み込みFontクラスを返します
        /// </para>
        /// </summary>
        /// <returns>Font オブジェクト</returns>
        public static Font getFont()
        {
            Font return_font = StandardFonts.Times;
            try
            {
                return_font = Font.FromFile(Path.Combine("Resources", "Fonts", "ipag.ttc"));
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message.ToString());
                Console.WriteLine("フォントを " + return_font.FontFamilyName.ToString() + " に設定します");
            }
            return return_font;
        }
    }
}
